
import { createRoot } from 'react-dom/client'
import { BrowserRouter } from 'react-router-dom'
import App from './App.tsx'
import './index.css'
import './styles/animationClasses.css'

// Register service worker for better caching (only in production)
if ('serviceWorker' in navigator && import.meta.env.MODE === 'production') {
  window.addEventListener('load', async () => {
    try {
      // Unregister any existing service workers first to ensure clean registration
      await navigator.serviceWorker.getRegistrations().then(function(registrations) {
        for(let registration of registrations) {
          console.log('Unregistering old service worker');
          registration.unregister();
        } 
      });
      
      // Register new service worker
      const registration = await navigator.serviceWorker.register('/serviceWorker.js', {
        scope: '/',
        updateViaCache: 'none'  // Never use cached version of service worker
      });
      console.log('ServiceWorker registration successful with scope: ', registration.scope);
      
      // Check for updates immediately
      registration.update().catch(err => {
        console.warn('Initial update check failed:', err);
      });
      
      // Set up update detection
      registration.addEventListener('updatefound', () => {
        const newWorker = registration.installing;
        if (newWorker) {
          newWorker.addEventListener('statechange', () => {
            console.log('Service worker state changed:', newWorker.state);
            if (newWorker.state === 'installed' && navigator.serviceWorker.controller) {
              // New content is available, show refresh notification
              console.log('New content available! Refresh to update.');
              // Here you could show a UI notification to the user
            }
          });
        }
      });
      
      // Check for updates periodically
      setInterval(() => {
        console.log('Checking for service worker updates...');
        registration.update().catch(err => {
          console.warn('Periodic update check failed:', err);
        });
      }, 60 * 60 * 1000); // Check every hour
      
    } catch (error) {
      console.error('ServiceWorker registration failed: ', error);
    }
  });
  
  // Listen for page refreshes and controller changes
  navigator.serviceWorker.addEventListener('controllerchange', () => {
    console.log('New service worker controller, refreshing content...');
  });
}

// For debugging sitemap and robots.txt issues
if (import.meta.env.DEV) {
  const checkFile = async (url: string) => {
    try {
      const response = await fetch(url, {
        cache: 'no-store',
        headers: {
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache'
        }
      });
      console.log(`${url} status:`, response.status, response.ok ? 'OK' : 'Not Found');
      if (response.ok) {
        const content = await response.text();
        console.log(`${url} length:`, content.length);
      }
    } catch (error) {
      console.error(`Error checking ${url}:`, error);
    }
  };
  
  // Check critical SEO files
  checkFile('/robots.txt');
  checkFile('/sitemap-index.xml');
  checkFile('/sitemap.xml');
}

// Mount the app - use the standard pattern for React 18 with error boundary
const rootElement = document.getElementById("root");
  
if (rootElement) {
  try {
    const root = createRoot(rootElement);
    root.render(
      <BrowserRouter>
        <App />
      </BrowserRouter>
    );
  } catch (error) {
    console.error("Error rendering application:", error);
    if (rootElement) {
      rootElement.innerHTML = `
        <div style="padding: 20px; text-align: center; font-family: system-ui, sans-serif;">
          <h1>Temporarily Unavailable</h1>
          <p>We're performing maintenance on our site. Please check back in a few minutes.</p>
        </div>
      `;
    }
  }
} else {
  console.error("Root element not found");
}

// Detect if the browser has prefers-reduced-motion enabled
const prefersReducedMotion = window.matchMedia('(prefers-reduced-motion: reduce)').matches;

// Conditionally disable animations if reduced motion is preferred
if (prefersReducedMotion) {
  document.documentElement.classList.add('no-animations');
}

// Add listener for font loading to avoid layout shifts
if (document.fonts && document.fonts.ready) {
  document.fonts.ready.then(() => {
    document.documentElement.classList.add('fonts-loaded');
  }).catch(() => {
    // Fallback if fonts fail to load
    setTimeout(() => {
      document.documentElement.classList.add('fonts-loaded');
    }, 1000);
  });
}
